@@ -18,10 +18,34 @@
  * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
  * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES RELAT-
  * ING TO THE SOFTWARE.
+ *
+ * $Id: matrix.c,v 1.8 1997/05/11 05:33:06 lister Exp $
  */
 
 #include "../../src/Matrix.h"
 
+void
+labelCB( mw, cd, cb )
+Widget mw;
+XtPointer cd;
+XtPointer cb;
+{
+    XbaeMatrixLabelActivateCallbackStruct *cbs =
+	( XbaeMatrixLabelActivateCallbackStruct * )cb;
+    int row, column;
+
+    if( cbs->row_label )
+	if( XbaeMatrixIsRowSelected( mw, cbs->row ) )
+	    XbaeMatrixDeselectRow( mw, cbs->row );
+	else
+	    XbaeMatrixSelectRow( mw, cbs->row );
+    else
+	if( XbaeMatrixIsColumnSelected( mw, cbs->column ) )
+	    XbaeMatrixDeselectColumn( mw, cbs->column );
+	else
+	    XbaeMatrixSelectColumn( mw, cbs->column );
+}
+
 /*
  * Simple example of loaded Matrix
  */
@@ -52,6 +76,9 @@ char *argv[];
 				 xbaeMatrixWidgetClass, toplevel,
 				 NULL);
 
+    XtAddCallback( mw, XmNlabelActivateCallback, ( XtCallbackProc )labelCB,
+		   NULL );
+    
     XtRealizeWidget(toplevel);
     XtAppMainLoop(app);
     
