@@ -20,6 +20,8 @@
  * ING TO THE SOFTWARE.
  *
  * ClipWidget Author: Andrew Wason, Bellcore, aw@bae.bellcore.com
+ *
+ * $Id: Clip.c,v 1.9 1997/02/08 03:08:20 lister Exp $
  */
 
 /*
@@ -46,12 +48,12 @@ static XtResource resources[] = {
 /*
  * Declaration of methods
  */
-static void ClassPartInitialize P(( XbaeClipWidgetClass cwc ));
-static void Realize P(( XbaeClipWidget cw, XtValueMask *valueMask,
-			XSetWindowAttributes *attributes ));
-static void Redisplay P(( XbaeClipWidget cw, XEvent *event, Region region ));
-static void Resize P(( Widget w ));
-static void Redraw P(( XbaeClipWidget cw ));
+static void ClassPartInitialize P(( XbaeClipWidgetClass ));
+static void Realize P(( XbaeClipWidget, XtValueMask *,
+			XSetWindowAttributes * ));
+static void Redisplay P(( XbaeClipWidget, XEvent *, Region ));
+static void Resize P(( Widget ));
+static void Redraw P(( XbaeClipWidget ));
 
 /*
  * Public convenience function
@@ -61,12 +63,11 @@ void XbaeClipRedraw P(( Widget w ));
 /*
  * Clip actions
  */
-static void FocusInACT P(( XbaeClipWidget cw, XEvent *event, String *params,
-			   Cardinal *nparams ));
+static void FocusInACT P(( XbaeClipWidget, XEvent *, String *, Cardinal * ));
 
 static XtActionsRec actions[] =
 {
-    {"FocusIn", ( XtActionProc )FocusInACT},
+    {"FocusIn", (XtActionProc)FocusInACT},
 };
 
 XbaeClipClassRec xbaeClipClassRec = {
@@ -76,11 +77,11 @@ XbaeClipClassRec xbaeClipClassRec = {
 	/* class_name		*/ "XbaeClip",
 	/* widget_size		*/ sizeof(XbaeClipRec),
 	/* class_initialize	*/ NULL,
-	/* class_part_initialize*/ ( XtWidgetClassProc )ClassPartInitialize,
+	/* class_part_initialize*/ (XtWidgetClassProc)ClassPartInitialize,
 	/* class_inited		*/ False,
 	/* initialize		*/ NULL,
 	/* initialize_hook	*/ NULL,
-	/* realize		*/ ( XtRealizeProc )Realize,
+	/* realize		*/ (XtRealizeProc)Realize,
 	/* actions		*/ actions,
 	/* num_actions		*/ XtNumber(actions),
 	/* resources		*/ resources,
@@ -94,7 +95,7 @@ XbaeClipClassRec xbaeClipClassRec = {
 	/* visible_interest	*/ False,
 	/* destroy		*/ NULL,
 	/* resize		*/ Resize,
-	/* expose		*/ ( XtExposeProc )Redisplay,
+	/* expose		*/ (XtExposeProc)Redisplay,
 	/* set_values		*/ NULL,
 	/* set_values_hook	*/ NULL,
 	/* set_values_almost	*/ XtInheritSetValuesAlmost,
@@ -119,7 +120,7 @@ XbaeClipClassRec xbaeClipClassRec = {
     },
     /* clip_class fields */
     {
-	/* redraw		*/ ( XbaeClipRedrawProc )Redraw,
+	/* redraw		*/ (XbaeClipRedrawProc)Redraw,
 	/* extension		*/ NULL,
     }
 };
@@ -188,10 +189,7 @@ Redraw(cw)
 XbaeClipWidget cw;
 {
     if (XtIsRealized((Widget)cw))
-	XClearArea(XtDisplay(cw), XtWindow(cw),
-		   0, 0,
-		   0 /*Full Width*/, 0 /*Full Height*/,
-		   True);
+	XClearArea(XtDisplay(cw), XtWindow(cw), 0, 0, 0, 0, True);
 }
 
 /*
@@ -212,7 +210,6 @@ Widget w;
     if (XtIsRealized(w))
 	(*((XbaeClipWidgetClass) XtClass(w))->clip_class.redraw)
 	    ((Widget)w);
-	    /* KPB ((XbaeClipWidget)w); */
 }
 
 /* ARGSUSED */
