@@ -20,6 +20,8 @@
  * ING TO THE SOFTWARE.
  *
  * ClipWidget Author: Andrew Wason, Bellcore, aw@bae.bellcore.com
+ *
+ * $Id: ClipP.h,v 1.8 1997/02/08 03:08:21 lister Exp $
  */
 
 /*
@@ -29,10 +31,6 @@
 #ifndef _Xbae_ClipP_h
 #define _Xbae_ClipP_h
 
-/*
- *  "@(#)ClipP.h	4.1 3/19/95"
- */
-
 #if XmVersion == 1001
 #include <Xm/XmP.h>
 #else
@@ -48,7 +46,7 @@
 extern "C" {
 #endif
 
-typedef void (*XbaeClipRedrawProc) P(( Widget w ));
+typedef void (*XbaeClipRedrawProc) P(( Widget ));
 
 #if defined (__cplusplus) || defined(c_plusplus)
 }
