@@ -1,5 +1,7 @@
 /*
  * Copyright(c) 1992 Bell Communications Research, Inc. (Bellcore)
+ * Copyright(c) 1995-97 Andrew Lister
+ *
  *                        All rights reserved
  * Permission to use, copy, modify and distribute this material for
  * any purpose and without fee is hereby granted, provided that the
@@ -21,6 +23,7 @@
  *
  * MatrixWidget Author: Andrew Wason, Bellcore, aw@bae.bellcore.com
  *
+ * $Id: Matrix.h,v 1.14 1997/05/03 01:04:33 lister Exp $
  */
 
 #ifndef _Xbae_Matrix_h
@@ -34,6 +37,23 @@
 #include <X11/Core.h>
 #include <Xbae/patchlevel.h>
 
+/*
+ * A few definitions we like to use, but those with R4 won't have.
+ * From Xfuncproto.h in R5.
+ */
+
+#ifndef XlibSpecificationRelease
+# ifndef _XFUNCPROTOBEGIN
+#   ifdef __cplusplus                      /* for C++ V2.0 */
+#     define _XFUNCPROTOBEGIN extern "C" {   /* do not leave open across includes */
+#     define _XFUNCPROTOEND }
+#   else
+#     define _XFUNCPROTOBEGIN
+#     define _XFUNCPROTOEND
+#   endif
+# endif /* _XFUNCPROTOBEGIN */
+#endif
+
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -44,18 +64,21 @@ extern "C" {
  * allowColumnResize	ColumnResize		Boolean		False
  * altRowCount		AltRowCount		int		1
  * boldLabels		BoldLabels		Boolean		False
- * cellBackgrounds	Colors			PixelTable	NULL ***
+ * buttonLabels		ButtonLabels		Boolean		False
+ * buttonLabelBackground Color			Pixel		dynamic
+ * cellBackgrounds	Colors			PixelTable	NULL
  * cellHighlightThickness HighlightThickness	HorizontalDimension 2
  * cellMarginHeight	MarginHeight		VerticalDimension   5
  * cellMarginWidth	MarginWidth		HorizontalDimension 5
  * cells		Cells			StringTable	NULL
- * cellShadowThickness	ShadowThickness		HorizontalDimension 2
+ * cellShadowThickness	ShadowThickness		Dimension	2
  * cellShadowType	ShadowType		unsigned char	SHADOW_OUT
  * cellShadowTypes	CellShadowTypes		ShadowTypeTable NULL
  * cellUserData		CellUserData		UserDataTable	NULL
  * clipWindow		XmCClipWindow		Widget		NULL (get only)
  * colors		Colors			PixelTable	NULL
  * columnAlignments	Alignments		AlignmentArray	dynamic
+ * columnButtonLabels	ButtonLabels		BooleanArray	NULL
  * columnLabelAlignments Alignments		AlignmentArray	dynamic
  * columnLabelColor	Color			Pixel		dynamic
  * columnLabels		Labels			StringArray	NULL
@@ -81,6 +104,7 @@ extern "C" {
  * horizontalScrollBarDisplayPolicy
  *			XmCMatrixScrollBarDisplayPolicy
  *						unsigned char	AS_NEEDED
+ * labelActivateCallback Callback		Callback	NULL
  * leaveCellCallback	Callback		Callback	NULL
  * leftColumn           LeftColumn              int             0
  * modifyVerifyCallback	Callback		Callback	NULL
@@ -89,6 +113,7 @@ extern "C" {
  * resizeCallback	Callback		Callback	NULL
  * resizeColumnCallback	Callback		Callback	NULL
  * reverseSelect	reverseSelect		Boolean		False
+ * rowButtonLabels		ButtonLabels		BooleanArray	NULL
  * rowLabelAlignment	Alignment		Alignment	XmALIGNMENT_END
  * rowLabelColor	Color			Pixel		dynamic
  * rowLabelWidth	RowLabelWidth		Short		dynamic
@@ -105,11 +130,13 @@ extern "C" {
  * shadowType		ShadowType		unsigned char	SHADOW_OUT
  * textBackground	Backgound		Pixel   	dynamic
  * textField		TextField		Widget		NULL (get only)
+ * textShadowThickness	TextShadowThickness	Dimension	0
  * textTranslations	Translations		TranslationTable dynamic
  * topRow		TopRow			int		0
  * trailingFixedColumns	TrailingFixedColumns	Dimension	0
  * trailingFixedRows	TrailingFixedRows	Dimension	0
  * traverseCellCallback	Callback		Callback	NULL
+ * traverseFixedCells	TraverseFixedCells	Boolean		False
  * verticalScrollBar	VerticalScrollBar	Widget		NULL (get only)
  * verticalScrollBarDisplayPolicy
  *			XmCMatrixScrollBarDisplayPolicy
@@ -122,6 +149,8 @@ extern "C" {
 #define XmNallowColumnResize		"allowColumnResize"
 #define XmNaltRowCount			"altRowCount"
 #define XmNboldLabels			"boldLabels"
+#define XmNbuttonLabels			"buttonLabels"
+#define XmNbuttonLabelBackground	"buttonLabelBackground"
 #define XmNcellBackgrounds		"cellBackgrounds"
 #define XmNcellHighlightThickness	"cellHighlightThickness"
 #define XmNcellMarginHeight		"cellMarginHeight"
@@ -136,7 +165,9 @@ extern "C" {
 #define XmNcells			"cells"
 #define XmNcolors			"colors"
 #define XmNcolumnAlignments		"columnAlignments"
+#define XmNcolumnButtonLabels		"columnButtonLabels"
 #define XmNcolumnLabelAlignments	"columnLabelAlignments"
+#define XmNcolumnLabelBackground	"columnLabelBackground"
 #define XmNcolumnLabelColor		"columnLabelColor"
 #define XmNcolumnLabels			"columnLabels"
 #define XmNcolumnMaxLengths		"columnMaxLengths"
@@ -152,20 +183,27 @@ extern "C" {
 #define XmNfixedRows			"fixedRows"
 #define XmNgridLineColor		"gridLineColor"
 #define XmNgridType			"gridType"
+#if XmVersion >= 1002
 #define XmNhighlightedCells		"highlightedCells"
+#endif
 #define XmNhorizontalScrollBarDisplayPolicy "horizontalScrollBarDisplayPolicy"
+#define XmNlabelActivateCallback	"labelActivateCallback"
 #define XmNlabelFont			"labelFont"
 #define XmNleaveCellCallback		"leaveCellCallback"
 #define XmNleftColumn			"leftColumn"
 #define XmNoddRowBackground		"oddRowBackground"
+#if XmVersion > 1001
 #define XmNprocessDragCallback		"processDragCallback"
+#endif
 #ifndef XmNresizeCallback
 #define XmNresizeCallback               "resizeCallback"
 #endif
 #define XmNresizeColumnCallback		"resizeColumnCallback"
 #define XmNreverseSelect		"reverseSelect"
+#define XmNrowButtonLabels		"rowButtonLabels"
 #define XmNrowLabelAlignment		"rowLabelAlignment"
 #define XmNrowLabelWidth		"rowLabelWidth"
+#define XmNrowLabelBackground		"rowLabelBackground"
 #define XmNrowLabelColor		"rowLabelColor"
 #define XmNrowLabels			"rowLabels"
 #define XmNrowShadowTypes		"rowShadowTypes"
@@ -181,7 +219,9 @@ extern "C" {
 #define XmNtrailingFixedColumns		"trailingFixedColumns"
 #define XmNtrailingFixedRows		"trailingFixedRows"
 #define XmNleftColumn			"leftColumn"
+#define XmNtextShadowThickness		"textShadowThickness"
 #define XmNtraverseCellCallback		"traverseCellCallback"
+#define XmNtraverseFixedCells		"traverseFixedCells"
 #define XmNverticalScrollBarDisplayPolicy "verticalScrollBarDisplayPolicy"
 #define XmNvisibleColumns		"visibleColumns"
 #define XmNvisibleRows			"visibleRows"
@@ -191,6 +231,7 @@ extern "C" {
 #define XmCAlignments			"Alignments"
 #define XmCAltRowCount			"AltRowCount"
 #define XmCBoldLabels			"BoldLabels"
+#define XmCButtonLabels			"ButtonLabels"
 #define XmCCells			"Cells"
 #define XmCCellShadowTypes		"CellShadowTypes"
 #define XmCCellUserData			"CellUserData"
@@ -205,7 +246,9 @@ extern "C" {
 #define XmCFixedColumns			"FixedColumns"
 #define XmCFixedRows			"FixedRows"
 #define XmCGridType			"GridType"
+#if XmVersion >= 1002
 #define XmCHighlightedCells		"HighlightedCells"
+#endif
 #define XmCLabels			"Labels"
 #define XmCLeftColumn			"LeftColumn"
 #define XmCMatrixScrollBarDisplayPolicy	"MatrixScrollBarDisplayPolicy"
@@ -216,6 +259,8 @@ extern "C" {
 #define XmCShadowTypes			"ShadowTypes"
 #define XmCTextBackground		"TextBackground"
 #define XmCTextField			"TextField"
+#define XmCTextShadowThickness		"TextShadowThickness"
+#define XmCTraverseFixedCells		"TraverseFixedCells"
 #define XmCTopRow			"TopRow"
 #define XmCTrailingFixedColumns		"TrailingFixedColumns"
 #define XmCTrailingFixedRows		"TrailingFixedRows"
@@ -223,16 +268,22 @@ extern "C" {
 #define XmCVisibleColumns		"VisibleColumns"
 #define XmCVisibleRows			"VisibleRows"
 
-#if XmVersion == 1001
+#ifndef XmRStringArray
 #define XmRStringArray			"StringArray"
 #endif
 
+#ifndef XmRBooleanArray
+#define XmRBooleanArray			"BooleanArray"
+#endif
+
 #define XmRAlignmentArray		"AlignmentArray"
 #define XmRBooleanTable			"BooleanTable"
 #define XmRCellTable			"CellTable"
 #define XmRWidgetTable			"WidgetTable"
 #define XmRGridType			"GridType"
+#if XmVersion >= 1002
 #define XmRHighlightTable		"HighlightTable"
+#endif
 #define XmRMatrixScrollBarDisplayPolicy "MatrixScrollBarDisplayPolicy"
 #define XmRMaxLengthArray		"MaxLengthArray"
 #define XmRPixelTable			"PixelTable"
@@ -275,7 +326,7 @@ typedef struct _XbaeMatrixRec *XbaeMatrixWidget;
 #ifndef XlibSpecificationRelease
 # ifndef _XFUNCPROTOBEGIN
 #   ifdef __cplusplus                      /* for C++ V2.0 */
-#     define _XFUNCPROTOBEGIN extern "C" {   /* do not leave open across includes */
+#     define _XFUNCPROTOBEGIN extern "C" {
 #     define _XFUNCPROTOEND }
 #   else
 #     define _XFUNCPROTOBEGIN
@@ -291,90 +342,81 @@ typedef struct _XbaeMatrixRec *XbaeMatrixWidget;
  */
 _XFUNCPROTOBEGIN
 
-extern void XbaeMatrixAddColumns P(( Widget w, int position, String *cols,
-				     String *labels, short *widths,
-				     int *max_lengths,
-				     unsigned char *alignments,
-				     unsigned char *label_alignments,
-				     Pixel *colors, int num_columns ));
-extern void XbaeMatrixAddRows P(( Widget w, int position, String *rows,
-				  String *labels, Pixel *colors,
-				  int num_rows ));
-extern void XbaeMatrixCancelEdit P(( Widget w, Boolean unmap ));
-extern Boolean XbaeMatrixCommitEdit P(( Widget w, Boolean unmap ));
-extern void XbaeMatrixDeleteColumns P(( Widget w, int position,
-					int num_columns ));
-extern void XbaeMatrixDeleteRows P(( Widget w, int position, int num_rows ));
-extern void XbaeMatrixDeselectAll P(( Widget w ));
-extern void XbaeMatrixDeselectCell P(( Widget w, int row, int column ));
-extern void XbaeMatrixDeselectColumn P(( Widget w, int column ));
-extern void XbaeMatrixDeselectRow P(( Widget w, int row ));
-extern void XbaeMatrixEditCell P(( Widget w, int row, int column ));
-extern void XbaeMatrixFirstSelectedCell P(( Widget w, int *row, int *column ));
-extern int XbaeMatrixFirstSelectedColumn P(( Widget w ));
-extern int XbaeMatrixFirstSelectedRow P(( Widget w ));
-extern String XbaeMatrixGetCell P(( Widget w, int row, int column ));
-extern XtPointer XbaeMatrixGetCellUserData P(( Widget w, int row,
-					       int column ));
-extern XtPointer XbaeMatrixGetColumnUserData P(( Widget w, int column ));
-extern void XbaeMatrixGetCurrentCell P(( Widget	w, int *row, int *column ));
-extern int XbaeMatrixGetEventRowColumn P(( Widget w, XEvent *event,
-					   int *row, int *column ));
+extern void XbaeMatrixAddColumns P(( Widget, int, String *, String *, short *,
+				     int *, unsigned char *, unsigned char *,
+				     Pixel *, int ));
+extern void XbaeMatrixAddRows P(( Widget,  int , String *, String *,
+				  Pixel *, int ));
+extern void XbaeMatrixCancelEdit P(( Widget, Boolean ));
+extern Boolean XbaeMatrixCommitEdit P(( Widget, Boolean ));
+extern void XbaeMatrixDeleteColumns P(( Widget, int, int ));
+extern void XbaeMatrixDeleteRows P(( Widget, int, int ));
+extern void XbaeMatrixDeselectAll P(( Widget ));
+extern void XbaeMatrixDeselectCell P(( Widget, int, int ));
+extern void XbaeMatrixDeselectColumn P(( Widget, int ));
+extern void XbaeMatrixDeselectRow P(( Widget, int ));
+extern void XbaeMatrixEditCell P(( Widget, int, int ));
+extern void XbaeMatrixFirstSelectedCell P(( Widget, int *, int * ));
+extern int XbaeMatrixFirstSelectedColumn P(( Widget ));
+extern int XbaeMatrixFirstSelectedRow P(( Widget ));
+extern String XbaeMatrixGetCell P(( Widget, int, int ));
+extern XtPointer XbaeMatrixGetCellUserData P(( Widget, int, int ));
+extern XtPointer XbaeMatrixGetColumnUserData P(( Widget, int ));
+extern void XbaeMatrixGetCurrentCell P(( Widget, int *, int * ));
+extern int XbaeMatrixGetEventRowColumn P(( Widget, XEvent *, int *, int * ));
+extern Boolean XbaeMatrixEventToXY P(( Widget, XEvent *, int *, int * ));
+extern Boolean XbaeMatrixRowColToXY P(( Widget, int, int, int *, int * ));
 extern int XbaeMatrixGetNumSelected P(( Widget ));
-extern XtPointer XbaeMatrixGetRowUserData P(( Widget w, int row ));
-extern Boolean XbaeMatrixIsCellSelected P(( Widget w, int row, int column ));
-extern Boolean XbaeMatrixIsColumnSelected P(( Widget w, int column ));
-extern Boolean XbaeMatrixIsRowSelected P(( Widget w, int row ));
-extern void XbaeMatrixRefresh P(( Widget w ));
-extern void XbaeMatrixSelectAll P(( Widget w ));
-extern void XbaeMatrixSelectCell P(( Widget w, int row, int column ));
-extern void XbaeMatrixSelectColumn P(( Widget w, int column ));
-extern void XbaeMatrixSelectRow P(( Widget w, int row ));
-extern void XbaeMatrixHighlightCell P(( Widget w, int row, int column ));
-extern void XbaeMatrixHighlightRow P(( Widget w, int row ));
-extern void XbaeMatrixHighlightColumn P(( Widget w, int column ));
-extern void XbaeMatrixUnhighlightCell P(( Widget w, int row, int column ));
-extern void XbaeMatrixUnhighlightRow P(( Widget w, int row ));
-extern void XbaeMatrixUnhighlightColumn P(( Widget w, int column ));
-extern void XbaeMatrixUnhighlightAll P(( Widget w ));
-extern void XbaeMatrixSetCell P(( Widget w, int row, int column,
-				  const String value ));
-extern void XbaeMatrixSetCellBackground P(( Widget w, int row, int column,
-					    Pixel color ));
-extern void XbaeMatrixSetCellColor P(( Widget w, int row, int column,
-				       Pixel color ));
-extern void XbaeMatrixSetCellUserData P(( Widget w, int row, int column,
-					  XtPointer data ));
-#if CELL_WIDGETS
-extern void XbaeMatrixSetCellWidget P(( Widget w, int row, int column,
-					Widget widget ));
+extern XtPointer XbaeMatrixGetRowUserData P(( Widget, int ));
+extern Boolean XbaeMatrixIsCellSelected P(( Widget, int, int ));
+extern Boolean XbaeMatrixIsColumnSelected P(( Widget, int ));
+extern Boolean XbaeMatrixIsRowSelected P(( Widget, int ));
+extern void XbaeMatrixRefresh P(( Widget ));
+extern void XbaeMatrixRefreshCell P(( Widget, int, int ));
+extern void XbaeMatrixSelectAll P(( Widget ));
+extern void XbaeMatrixSelectCell P(( Widget, int, int ));
+extern void XbaeMatrixSelectColumn P(( Widget, int ));
+extern void XbaeMatrixSelectRow P(( Widget, int ));
+#if XmVersion >= 1002
+extern void XbaeMatrixHighlightCell P(( Widget, int, int ));
+extern void XbaeMatrixHighlightRow P(( Widget, int ));
+extern void XbaeMatrixHighlightColumn P(( Widget, int ));
+extern void XbaeMatrixUnhighlightCell P(( Widget, int, int ));
+extern void XbaeMatrixUnhighlightRow P(( Widget, int ));
+extern void XbaeMatrixUnhighlightColumn P(( Widget, int ));
+extern void XbaeMatrixUnhighlightAll P(( Widget ));
 #endif
-extern void XbaeMatrixSetColumnBackgrounds P(( Widget w, int position,
-					       Pixel *colors,
-					       int num_colors ));
-extern void XbaeMatrixSetColumnColors P(( Widget w, int position,
-					  Pixel *colors, int num_colors ));
-extern void XbaeMatrixSetColumnUserData P(( Widget w, int column,
-					    XtPointer data ));
-extern void XbaeMatrixSetRowBackgrounds P(( Widget w, int position,
-					    Pixel *colors, int num_colors ));
-extern void XbaeMatrixSetRowColors P(( Widget w, int position, Pixel *colors,
-				       int num_colors ));
-extern void XbaeMatrixSetRowUserData P(( Widget w, int row, XtPointer data ));
-
-extern int XbaeMatrixVisibleColumns P(( Widget w ));
-extern int XbaeMatrixVisibleRows P(( Widget w ));
-extern int XbaeMatrixNumColumns P(( Widget w ));
-extern int XbaeMatrixNumRows P(( Widget w ));
-extern int XbaeMatrixXToCol P(( Widget w, int x ));
-extern int XbaeMatrixYToRow P(( Widget w, int y ));
-
-extern void XbaeMatrixDisableRedisplay P(( Widget w ));
-extern void XbaeMatrixEnableRedisplay P(( Widget w, Boolean redisplay));
+extern void XbaeMatrixSetCell P(( Widget, int, int, const String ));
+extern void XbaeMatrixSetCellBackground P(( Widget, int, int, Pixel ));
+extern void XbaeMatrixSetCellColor P(( Widget, int, int, Pixel ));
+extern void XbaeMatrixSetCellUserData P(( Widget, int, int, XtPointer ));
+#if CELL_WIDGETS
+extern void XbaeMatrixSetCellWidget P(( Widget, int, int, Widget ));
+#endif
+extern void XbaeMatrixSetColumnBackgrounds P(( Widget, int, Pixel *, int ));
+extern void XbaeMatrixSetColumnColors P(( Widget, int, Pixel *, int ));
+extern void XbaeMatrixSetColumnUserData P(( Widget, int, XtPointer ));
+extern void XbaeMatrixSetRowBackgrounds P(( Widget, int, Pixel *, int ));
+extern void XbaeMatrixSetRowColors P(( Widget, int , Pixel *, int ));
+extern void XbaeMatrixSetRowUserData P(( Widget, int, XtPointer ));
+extern int XbaeMatrixVisibleColumns P(( Widget ));
+extern int XbaeMatrixVisibleRows P(( Widget ));
+extern int XbaeMatrixNumColumns P(( Widget ));
+extern int XbaeMatrixNumRows P(( Widget ));
+extern void XbaeMatrixDisableRedisplay P(( Widget ));
+extern void XbaeMatrixEnableRedisplay P(( Widget, Boolean ));
+extern void XbaeMatrixMakeCellVisible P(( Widget, int, int ));
+extern Boolean XbaeMatrixIsRowVisible P(( Widget, int ));
+extern Boolean XbaeMatrixIsColumnVisible P(( Widget, int ));
+extern Boolean XbaeMatrixIsCellVisible P(( Widget, int, int ));
+extern void XbaeMatrixVisibleCells P(( Widget, int *, int *, int *, int * ));
+extern String XbaeMatrixGetColumnLabel P(( Widget, int ));
+extern String XbaeMatrixGetRowLabel P(( Widget, int ));
+extern void XbaeMatrixSetColumnLabel P(( Widget, int, String ));
+extern void XbaeMatrixSetRowLabel P(( Widget, int, String ));
 
 _XFUNCPROTOEND
 
-#ifndef XBAE_NO_EXTRA_TYPES
 typedef unsigned char	Alignment;
 typedef Alignment *	AlignmentArray;
 typedef String *	StringTable;
@@ -382,7 +424,6 @@ typedef short 		Width;
 typedef Width *		WidthArray;
 typedef int 		MaxLength;
 typedef MaxLength *	MaxLengthArray;
-#endif
 
 /*
  * cell shadow types
@@ -413,8 +454,11 @@ enum
 /*
  * Enumeration for type of a cell
  */
-typedef enum { FixedCell, NonFixedCell } CellType;
+typedef enum {
+    FixedCell, NonFixedCell, RowLabelCell, ColumnLabelCell
+} CellType;
 
+#if XmVersion >= 1002
 /*
  * Enumeration for highlight reason/location
  */
@@ -429,7 +473,7 @@ enum {
     UnhighlightColumn	= 0x0040,
     UnhighlightAll	= UnhighlightCell | UnhighlightRow | UnhighlightColumn
 };
-
+#endif
 
 /*
  * Callback reasons.  Try to stay out of range of the Motif XmCR_* reasons.
@@ -446,7 +490,8 @@ typedef enum _XbaeReasonType
     XbaeResizeReason,
     XbaeResizeColumnReason,
     XbaeDefaultActionReason,
-    XbaeProcessDragReason
+    XbaeProcessDragReason,
+    XbaeLabelActivateReason
 }
 XbaeReasonType;
 
@@ -480,6 +525,7 @@ typedef struct _XbaeMatrixEnterCellCallbackStruct
     XbaeReasonType reason;
     int row, column;
     Boolean select_text;
+    Boolean map;
     Boolean doit;
 }
 XbaeMatrixEnterCellCallbackStruct;
@@ -505,7 +551,6 @@ typedef struct _XbaeMatrixTraverseCellCallbackStruct
     int row, column;
     int next_row, next_column;
     int fixed_rows, fixed_columns;
-    int leading_fixed_rows, leading_fixed_columns;
     int trailing_fixed_rows, trailing_fixed_columns;
     int num_rows, num_columns;
     String param;
@@ -584,7 +629,7 @@ typedef struct _XbaeMatrixResizeColumnCallbackStruct
 }
 XbaeMatrixResizeColumnCallbackStruct;
 
-
+#if XmVersion > 1001
 /*
  * Struct passed to processDragCallback
  */
@@ -601,7 +646,7 @@ typedef struct _XbaeMatrixProcessDragCallbackStruct
     XEvent *event;
 }
 XbaeMatrixProcessDragCallbackStruct;
-
+#endif
 
 /*
  * Struct passed to defaultActionCallback
@@ -614,6 +659,20 @@ typedef struct _XbaeMatrixDefaultActionCallbackStruct
 }
 XbaeMatrixDefaultActionCallbackStruct;
 
+/*
+ * Struct passed to labelActivateCallback
+ */
+typedef struct _XbaeMatrixLabelActivateCallbackStruct
+{
+    XbaeReasonType reason;
+    XEvent *event;
+    Boolean row_label;
+    int row, column;
+    String label;
+}
+XbaeMatrixLabelActivateCallbackStruct;
+
+
 /* provide clean-up for those with R4 */
 #ifndef XlibSpecificationRelease
 # undef _Xconst
